########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 26/05/2009
# Modified on 	: 01/06/2009 (Output of results with lifted positions now only if now fault so no NA output)
#
# Input files 	: mm9ToMm8.over.chain (Downloaded from UCSC and stored in Input_Files directory)
#				  K_intron.txt (Output by tcl script INTRONS16_Kintrongene.tcl and stored in Intron_Data/RESULTS directory)
# 
# Output files  : K_intron_liftOver.txt (Stored in Output_Files directory)
#				  K_intron_liftOver.xls (Stored in Output_Files directory)
# 				  tempIN.bed (Temporary file generated for input to liftOver for each gene and deleted)
#				  tempOUT.bed (Temporary file generated as output by liftOver for each gene and deleted)
#				  tempERROR.bed (Temporary file generated as output by liftOver for each gene and deleted)
#
# This script converts the position of each intron in the mm9 build and uses the liftover software to convert it to a position in the mm8 build.
# The script takes the results file containing the substitution rates of each intron and uses the position of the intron defined by the bp at the ends 
# of the intron in mouse build mm9 and then uses the liftOver executable and chain file downloaded from UCSC to convert these positions to a position in
# mouse build mm8.
# For each intron with substitution rate data, the chromosomal location and positional limits of the intron are extracted from the data line and output
# in bed format to a temporary file. This file is then fed to the liftOver software which uses the over.chain file for the mm9 to mm8 lift to determine 
# the position of the intervening bit of sequence in mouse build mm8. The liftOver software then outputs either the position in mm8 or an error message
# in a bed format to temporary files tempOUT.bed and tempERROR.bed. 
# The presence or absence of any text in the contents of the tempERROR.bed file is used to determine whether the lift over was successfull (no 
# error text = successful)
# For successful lifts, the position of the intron in mouse build mm8 is extracted and then output to a new file together with the original position.
#
########################################################################################################################################################

package provide 20_liftoverINTRONS 1.0

proc 20_liftoverINTRONS {} {

	puts "-----------------------------------------------------------"
	puts "20             LIFTOVER mm9 -> mm8 - INTRONS               \n"

	 # Make variables global
	global contents
	
	# Move chain file to main directory
	cd Input_Files
	exec cp mm9ToMm8.over.chain ../mm9ToMm8.over.chain
	cd ..
	
	# Create new file for output results
	set outTXT [open K_intron_liftOver.txt w]
	set outXLS [open K_intron_liftOver.xls w]
	
	puts $outTXT "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	puts $outXLS "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
		
	# Create output file for liftOver error messages
	set outERROR [open liftoverErrors_Introns.txt w]

		
	# Extract original intron results
	cd Intron_Data/RESULTS
	openfile K_intron.txt
	cd ../..
	set contentsK_intron [split $contents \n]
	unset contents
	set contentsK_intron [lrange $contentsK_intron 1 end-1]
	set count 0
	foreach dataline $contentsK_intron {
		
		incr count
		if {[expr {$count %200}] == 0} {puts "*** $count of [llength $contentsK_intron] ***"}


		# Find unlifted position mm9
		set mm9chromosome chr[lindex $dataline 2]
		set mm9start [lindex $dataline 6]
		set mm9end [lindex $dataline 7]

		# Output unlifted position to tempIN.bed file
		set outIN [open tempIN.bed w]
		puts $outIN "$mm9chromosome\t$mm9start\t$mm9end"
		close $outIN
		unset outIN
	
		# ***** DO LIFT OVER ***** 
		catch {exec liftOver tempIN.bed mm9ToMm8.over.chain tempOUT.bed tempERROR.bed -minMatch=0.95} outputmsg
		unset outputmsg
		# ************************
		
		# Determine if the liftOver was successfull (Based on any output of text to tempERROR.bed file)
		openfile tempERROR.bed
		set contentsERROR $contents
		unset contents
		if {[regexp {[\w]} $contentsERROR all]} {
		
			puts "-----------------------------------------------------------\nLiftOver Error for ortholog [lindex $dataline 0]\n$mm9chromosome:$mm9start-$mm9end\n$contentsERROR"
			puts $outERROR "-----------------------------------------------------------\nError with liftOver for for ortholog [lindex $dataline 0]\nmm9 details : $mm9chromosome : $mm9start - $mm9end\nmm8 output : $contentsERROR"
			set mm8chromosome NA
			set mm8start NA
			set mm8end NA
						
		} else {
					
			# Extract contents from tempOUT.bed
			openfile tempOUT.bed
			set contentsOUT $contents
			unset contents
			regexp {chr([\w]+)\s+([0-9]+)\s+([0-9]+)} $contentsOUT all mm8chromosome mm8start mm8end
						
			# Check assumption about same chromosomes is true			
			if {[lindex $dataline 2] != $mm8chromosome} {
				puts "-----------------------------------------------------------\nLiftOver Error for ortholog [lindex $dataline 0]\n$mm9chromosome:$mm9start-$mm9end\n#Chromosomes differ between assemblies\n[lindex $dataline 1] != $mm8chromosome"
				puts $outERROR "-----------------------------------------------------------\nError with liftOver for ortholog [lindex $dataline 0]\nmm9 details : $mm9chromosome : $mm9start - $mm9end\nmm8 output : #Chromosomes differ between assemblies\n$contentsOUT"
				set mm8chromosome NA
				set mm8start NA
				set mm8end NA
				
			}
			unset contentsOUT all
			
		}
		unset contentsERROR
				
		# Send original results and lifted positions to output file
		puts $outTXT "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t$mm8start\t$mm8end\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]"
		puts $outXLS "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t$mm8start\t$mm8end\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]"

		# Unset variables
		unset mm9chromosome mm9start mm9end  	
		unset mm8chromosome mm8start mm8end


		# Remove temporary files for this dataline
		exec rm tempIN.bed
		exec rm tempOUT.bed
		exec rm tempERROR.bed

	}
	unset dataline contentsK_intron count
	
	
	# Close output files and move to Output_Files directory
	close $outTXT
	close $outXLS
	close $outERROR
	unset outTXT outXLS outERROR
	exec mv K_intron_liftOver.txt Output_Files/K_intron_liftOver.txt
	exec mv K_intron_liftOver.xls Output_Files/K_intron_liftOver.xls
	exec mv liftoverErrors_Introns.txt Output_Files/liftoverErrors_Introns.txt

	# Remove chain file from main directory
	exec rm mm9ToMm8.over.chain

    puts "\nVariables still open in liftoverINTRONS : [info local]"

return
}
