########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 12/06/2009
# Modified on 	: 
#
# Input files 	: 
#				  		  
# Output files  : 
#				  
# This script attempts to quantify the amount of rearrangement an autosome has undergone.
# The principle behind the scrip is as follows:
#		On an autosome, a pair of genes (out of the ortholog gene data set) are randomly selected.
# 		Chromosomal locations of each gene are identified from the input results file.
#		The orthologous genes to each of these genes are identified in the other species.
#		The chromosomal location of these orthologs is identifed.
#		If the two orthologs of the original pair of genes are on the same chromosome, no count is made.
#		If the two orthologs of the original pair of genes are on different chromosomes in the second species (they are by default on
#		the same chromosome in the first species) then a count is made.
#		By repeating this 10000 for each autosome, and counting the number of times the pair of orthologs are on different chromosomes,
#		a measure of rearrangement can be made. 
#		The more rearranged an autosome is, the more likely orthologs from two genes on that autosome will be on different autosomes 
#		in the other species.
# Rearrangements indices for the X and Y chromosome are irrelevant as the criteria for orthology identification was that the chromosomal
# class of each ortholog must be identical. Therefore, for X and Y, all orthologs will be on X and Y respectively and rearrangement
# indicies will be O for each.
#
########################################################################################################################################################



package provide 26_rearrangement_index 1.0

proc 26_rearrangement_index {} {
	
	puts "-----------------------------------------------------------"
	puts "26             GENERATING REARRANGEMENT INDEX              \n"
	
	# Make variables globale
	global contents
	
	
	
	# Initiate arrays for each species indexed by chromosome to store pairs of orthologous chromosomal locations
	puts "Initiating arrays for pairs of orthologous autosomes"
	for {set count 1} {$count <= 19} {incr count} {
		set MOUSE($count) {}
	}; unset count
	for {set count 1} {$count <= 20} {incr count} {
		set RAT($count) {}
	}; unset count

	# Extract contents of K_gene_RepTime_genic.txt to save genes and chromosomal locations in an array
	puts "Extracting chromosomal locations of each autosomal orthologous pair"
	cd Output_Files
	openfile K_gene_RepTime_genic.txt
	cd ..
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		
		set chr_mouse [lindex $dataline 1]
		set chr_rat [lindex $dataline 2]
		set start_mouse [lindex $dataline 6]
		set start_rat [lindex $dataline 10]
		
		# Append the extracted autosomes to either the MOUSE or RAT arrays
		if {$chr_mouse != "X" && $chr_mouse != "Y"} {
			lappend MOUSE($chr_mouse) "$chr_mouse $chr_rat"
			lappend RAT($chr_rat) "$chr_rat $chr_mouse"
		} 
		
		unset chr_mouse chr_rat start_mouse start_rat
	}
	unset dataline contents
  	






	# *********************************
	# RANDOMISATION FOR EACH CHROMOSOME
	# *********************************	
	
	# Set the number of repeats
	set repeats 10000


	# BASED ON MOUSE CHROMOSOME
	puts "Counting mouse ortholog pairs on different rat autosomes"
	foreach autosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19} {
		
#		puts "Mouse autosome $autosome"
		
		# Initiate scores for this autosome
		set sameMOUSEdifferentRAT($autosome) 0
		
		set repeatcount 1
		while {$repeatcount <= $repeats} {
			
#			puts "$repeatcount of $repeats"
			
			# Generate 2 different random numbers to index within the list
			set randomnumber1 [expr {int( [expr {[llength $MOUSE($autosome)] * rand() }])}]
			set randomnumber2 [expr {int( [expr {[llength $MOUSE($autosome)] * rand() }])}]
			while {$randomnumber1 == $randomnumber2} {
				set randomnumber2 [expr {int( [expr {[llength $MOUSE($autosome)] * rand() }])}]
			}

			# Extract orthologue pairs from list for this chromosome indexed by random numbers
			set orthologs1 [lindex $MOUSE($autosome) $randomnumber1]
			set orthologs2 [lindex $MOUSE($autosome) $randomnumber2]
		
			# Check that mouse chromosome is the same
			if {[lindex $orthologs1 0] != [lindex $orthologs2 0]} {
				puts "ERROR....Mouse chromosomal locations not the same: [lindex $orthologs1 0] & [lindex $orthologs2 0]. Exiting the program"
				exit
			}
			
			# CRITICAL BIT - ARE RAT CHROMOSOMES THE SAME?
			if {[lindex $orthologs1 1] != [lindex $orthologs2 1]} {
				incr sameMOUSEdifferentRAT($autosome)
			}			
			
			# Unset variables
			unset orthologs1 orthologs2
			unset randomnumber1 randomnumber2
			
			# Increase repeatcount
			incr repeatcount
		}
		unset repeatcount
	}
	unset autosome


	# BASED ON RAT CHROMOSOME
	puts "Counting rat ortholog pairs on different mouse autosomes"
	foreach autosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20} {
			
#		puts "Rat autosome $autosome"
	
		# Initiate scores for this autosome
		set sameRATdifferentMOUSE($autosome) 0
		
		set repeatcount 1
		while {$repeatcount <= $repeats} {

#			puts "$repeatcount of $repeats"

			# Generate 2 different random numbers to index within the list
			set randomnumber1 [expr {int( [expr {[llength $RAT($autosome)] * rand() }])}]
			set randomnumber2 [expr {int( [expr {[llength $RAT($autosome)] * rand() }])}]
			while {$randomnumber1 == $randomnumber2} {
				set randomnumber2 [expr {int( [expr {[llength $RAT($autosome)] * rand() }])}]
			}
		
			# Extract orthologue pairs from list for this chromosome indexed by random numbers
			set orthologs1 [lindex $RAT($autosome) $randomnumber1]
			set orthologs2 [lindex $RAT($autosome) $randomnumber2]
		
			# Check that rat chromosome is the same
			if {[lindex $orthologs1 0] != [lindex $orthologs2 0]} {
				puts "ERROR....Rat chromosomal locations not the same: [lindex $orthologs1 0] & [lindex $orthologs2 0]. Exiting the program"
				exit
			}

			# CRITICAL BIT - ARE MOUSE CHROMOSOMES THE SAME?
			if {[lindex $orthologs1 1] != [lindex $orthologs2 1]} {
				incr sameRATdifferentMOUSE($autosome) 1
			}
			
			# Unset variables
			unset orthologs1 orthologs2
			unset randomnumber1 randomnumber2
			
			# Increase repeatcount
			incr repeatcount
		}
		unset repeatcount
	}
	unset autosome

	
	
	
	
	
	
	
	
	
	
	# *******************************************************
	# Calculate rearrangement index for each species/autosome
	# *******************************************************
	
	puts "Generating rearrangement indices for each autosome in each species"
	
	# Create output file to store results
	set outTXT [open Rearrangement_Index_$repeats.txt w]
	set outXLS [open Rearrangement_Index_$repeats.xls w]
	puts $outTXT "Species\tAutosome\tNumber_Orthologues\tCount_Different_Chr\tRearrangement_index"
	puts $outXLS "Species\tAutosome\tNumber_Orthologues\tCount_Different_Chr\tRearrangement_index"

	# CALCULATE MOUSE REARRANGEMENT INDEX AND SEND RESULTS TO OUTPUT FILE
	foreach autosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19} {
		set index [expr {$sameMOUSEdifferentRAT($autosome) / [format %f $repeats]  }]
		puts $outTXT "Mouse\t$autosome\t[llength $MOUSE($autosome)]\t$sameMOUSEdifferentRAT($autosome)\t$index"
		puts $outXLS "Mouse\t$autosome\t[llength $MOUSE($autosome)]\t$sameMOUSEdifferentRAT($autosome)\t$index"
		unset index
	}
	unset autosome
	
	# CALCULATE RAT REARRANGEMENT INDEX AND SEND RESULTS TO OUTPUT FILE	
	foreach autosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20} {
		set index [expr {$sameRATdifferentMOUSE($autosome) / [format %f $repeats]  } ]
		puts $outTXT "Rat\t$autosome\t[llength $RAT($autosome)]\t$sameRATdifferentMOUSE($autosome)\t$index"
		puts $outXLS "Rat\t$autosome\t[llength $RAT($autosome)]\t$sameRATdifferentMOUSE($autosome)\t$index"
		unset index
	}
	unset autosome
	
	# Close output files and move to Output_Files directory
	close $outTXT
	close $outXLS
	unset outTXT outXLS

	exec mv Rearrangement_Index_$repeats.txt Output_Files/Rearrangement_Index_$repeats.txt
	exec mv Rearrangement_Index_$repeats.xls Output_Files/Rearrangement_Index_$repeats.xls


	# Unset variables
	unset repeats
	unset MOUSE RAT
	unset sameMOUSEdifferentRAT sameRATdifferentMOUSE


	puts "\nVariables open in rearrangement_index : [info local]"

return
}
