########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 28/11/2007 (As part of MMBIntrons work)
# Modified on 	: 14/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  14/05/2009 (Complete restructuring of script)				  
#				  15/05/2009 (Debugging to extract information from file names including spaces)
#				  15/05/2009 (Debugging to handle Y-linked annotations different (rat gene names different and no positions)
#				  05/06/2009 (Removal of introns with -1 error results)
#
# Input files 	: 
# 				  (Output by tcl script INTRONS15_concatenatechromosomes.tcl)
#				  
# Output files  : K_noncoding_out.tab (from Knc.pl script - deleted)
#				  K_intron.txt
#				  K_intron.xls
#
# This script calculates intronic substitution rates for chromosomes.
# It moves the Knc.pl script from the mmREPLICATION_tcl.tcl library to the ALIGNMENTS_final directory containing the aligned intronic sequences.
# The Knc.pl script is executed, generating the output file K_noncoding_out.tab.
# The K_noncoding_out.tab file is moved to the new RESULTS directory.
# The contents of the K_noncoding_out.tab file are reformated to remove file and sequence names.
# Separate chromosome, gene name (refseq ID), intron number, intron start and end position columns are added for each species.
# The reformated results are output into two new files within the new RESULTS directory.
# To avoid confusion, the K_noncoding_out.tab file is deleted.	  
#
########################################################################################################################################################


package provide INTRONS16_Kintron 1.0

proc INTRONS16_Kintron {} {
	
	puts "-----------------------------------------------------------"
	puts "16        CALCULATING SUBSTITUTION RATES - INTRONS         \n"

	# Make variables global
	global contents
	
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data
	
	# Create new RESULTS directory
	if {![file exists RESULTS]} {
		puts "No RESULTS directory was found"
		exec mkdir RESULTS
		puts "New RESTULTS directory was created"
	}
	
	puts "Preparation for Knc.pl script in alignment files directory"
	# Move perl Knc.pl script into the directory containing the final data set
	exec cp ../mmREPLICATION_tcl_library/Knc.pl ./ALIGNMENTS_final/Knc.pl
	
	# Move to ALIGNMENTS_final directory
	cd ALIGNMENTS_final

	# Remove any K_noncoding_out.tab files already in existance to prevent new results appending to current file
	if {[file exists K_noncoding_out.tab]} {
		exec rm K_noncoding_out.tab
	}
    
	puts "Executing Knc.pl script"
	# Execute Martins Knc.pl script to calculate substitution rates
	exec perl Knc.pl
	puts "Substitution rates calculated"
	
	# Remove Knc.pl script from directory
	exec rm Knc.pl
	
	# Move output K_noncoding_out.tab file to RESULTS directory
	exec mv K_noncoding_out.tab ../RESULTS/K_noncoding_out.tab

	# Return from CONCATENATE_chromosomes to Intron_Data directory
	cd ..
	puts "Tidied Knc.pl and output file from alignment files directory"


	puts "Reformating output results file"
	# Move to RESULTS directory
	cd RESULTS

	# Create a new output file to store results in
	set outTXT [open K_intron.txt w]
	set outXLS [open K_intron.xls w]

	puts $outTXT "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start\tMouse_End\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	puts $outXLS "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start\tMouse_End\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	
	# Reformat each line to include mouse chromosome, alignment length and number of introns rather than file or alignment names
	openfile K_noncoding_out.tab
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		
		set alignmentfilename [lindex $dataline 0]
		set mousename [lindex $dataline 3]
		set ratname [lindex $dataline 6]

		regexp {passed_Orth-([0-9]+)-Intron-([0-9]+)-Chr-([\w]+).nuc} $alignmentfilename all ortholog intronnumber chromosome
		
		if {$chromosome != "Y"} {
			regexp {mm:(NM_[0-9]+)_[0-9]+:chr([\w]+):([0-9]+)-([0-9]+)} $mousename all mouserefseq mousechromosome mousestart mouseend
			regexp {rn:(NM_[0-9]+)_[0-9]+:chr([\w]+):([0-9]+)-([0-9]+)} $ratname all ratrefseq ratchromosome ratstart ratend
		} else {
			regexp {mm:(NM_[0-9]+):chr([\w]+):([0-9]+)-([0-9]+)} $mousename all mouserefseq mousechromosome mousestart mouseend
			regexp {rn:(NM_[\w]+):chr([\w]+)} $ratname all ratrefseq ratchromosome
			set ratstart 0
			set ratend 0
		}
		
		if {[lindex $dataline 4] != -1 && [lindex $dataline 5] != -1 && [lindex $dataline 6] != -1 && [lindex $dataline 7] != -1} {
			puts $outTXT "$ortholog\t$intronnumber\t$mousechromosome\t$ratchromosome\t$mouserefseq\t$ratrefseq\t$mousestart\t$mouseend\t$ratstart\t$ratend\t[lindex $dataline end-7]\t[lindex $dataline end-6]\t[lindex $dataline end-5]\t[lindex $dataline end-4]\t[lindex $dataline end-3]\t[lindex $dataline end-2]\t[lindex $dataline end-1]\t[lindex $dataline end]"
			puts $outXLS "$ortholog\t$intronnumber\t$mousechromosome\t$ratchromosome\t$mouserefseq\t$ratrefseq\t$mousestart\t$mouseend\t$ratstart\t$ratend\t[lindex $dataline end-7]\t[lindex $dataline end-6]\t[lindex $dataline end-5]\t[lindex $dataline end-4]\t[lindex $dataline end-3]\t[lindex $dataline end-2]\t[lindex $dataline end-1]\t[lindex $dataline end]"
		}
		
		unset all alignmentfilename mousename ratname
		unset ortholog intronnumber chromosome
		unset mouserefseq mousechromosome mousestart mouseend
		unset ratrefseq ratchromosome ratstart ratend
	}
	unset dataline contents

	# Close new results file
	close $outTXT
	close $outXLS
	unset outTXT outXLS
	
	# Remove previous K_noncoding_out.tab file
	exec rm K_noncoding_out.tab
	
	# Return from RESULTS to Intron_Data directory
	cd ..
	# Return from Intron_Data directory to main directory
	cd ..

	puts "\nVariables open in INTRONS16_Kintron : [info local]"

return
}