########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 28/11/2007 (As part of MMBIntrons work)
# Modified on 	: 14/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  14/05/2009 (Complete restructuring of script)				  
#
# Input files 	: All Concat_Chr-$chromosome-Length-$alignmentlength.nuc files in CONCATENATE_chromosomes directory
# 				  (Output by tcl script INTRONS15_concatenatechromosomes.tcl)
#				  
# Output files  : K_noncoding_out.tab (from Knc.pl script - deleted)
#				  K_chromosome.txt
#				  K_chromosome.xls
#
# This script calculates intronic substitution rates for chromosomes.
# It moves the Knc.pl script from the mmREPLICATION_tcl.tcl library to the CONCATENATE_chromosomes directory containing the aligned chromosomal sequences.
# The Knc.pl script is executed, generating the output file K_noncoding_out.tab.
# The K_noncoding_out.tab file is moved to the new RESULTS directory.
# The contents of the K_noncoding_out.tab file are reformated to remove file and sequence names.
# Separate chromosome, alignment length and intron number columns are added.
# The reformated results are output into two new files within the new RESULTS directory.
# To avoid confusion, the K_noncoding_out.tab file is deleted.	  
#
########################################################################################################################################################

package provide INTRONS18_Kchromosome 1.0

proc INTRONS18_Kchromosome {} {
	
	puts "-----------------------------------------------------------"
	puts "18      CALCULATING SUBSTITUTION RATES - CHROMOSOMES       \n"

	# Make variables global
	global contents
	
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data

	
	# Create new RESULTS directory
	if {![file exists RESULTS]} {
		puts "No RESULTS directory was found"
		exec mkdir RESULTS
		puts "New RESTULTS directory was created"
	}
	
	puts "Preparation for Knc.pl script in alignment files directory"
	# Move perl Knc.pl script into the directory containing the final data set
	exec cp ../mmREPLICATION_tcl_library/Knc.pl ./CONCATENATE_chromosomes/Knc.pl
	
	# Move to CONCATENATE_chromosomes directory
	cd CONCATENATE_chromosomes

	# Remove any K_noncoding_out.tab files already in existance to prevent new results appending to current file
	if {[file exists K_noncoding_out.tab]} {
		exec rm K_noncoding_out.tab
	}
    
	puts "Executing Knc.pl script"
	# Execute Martins Knc.pl script to calculate substitution rates
	exec perl Knc.pl
	puts "Substitution rates calculated"
	
	# Remove Knc.pl script from directory
	exec rm Knc.pl
    
	# Move output K_noncoding_out.tab file to RESULTS directory
	exec mv K_noncoding_out.tab ../RESULTS/K_noncoding_out.tab

	# Return from CONCATENATE_chromosomes to Intron_Data directory
	cd ..
	puts "Tidied Knc.pl and output file from alignment files directory"


	puts "Reformating output results file"
	# Move to RESULTS directory
	cd RESULTS

	# Create a new output file to store results in
	set outTXT [open K_chromosome.txt w]
	set outXLS [open K_chromosome.xls w]

	puts $outTXT "Mouse_Chromosome\tAlignment_Length\tIntrons_Concatenated\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	puts $outXLS "Mouse_Chromosome\tAlignment_Length\tIntrons_Concatenated\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	
	# Reformat each line to include mouse chromosome, alignment length and number of introns rather than file or alignment names
	openfile K_noncoding_out.tab
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		
		set alignmentfilename [lindex $dataline 0]
		set mousename [lindex $dataline 1]

		regexp {Concat_Chr-(.+)-Length-([0-9]+)\.nuc} $alignmentfilename all chromosome alignmentlength
		regexp {>Mouse:#Introns-([0-9]+)} $mousename all numberofintrons
		
		if {[lindex $dataline 4] != -1 && [lindex $dataline 5] != -1 && [lindex $dataline 6] != -1 && [lindex $dataline 7] != -1} {
			puts $outTXT "$chromosome\t$alignmentlength\t$numberofintrons\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]"
			puts $outXLS "$chromosome\t$alignmentlength\t$numberofintrons\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]"
		}
		
		unset all alignmentfilename mousename
		unset chromosome alignmentlength numberofintrons
	}
	unset dataline contents

	# Close new results file
	close $outTXT
	close $outXLS
	unset outTXT outXLS
	
	# Remove previous K_noncoding_out.tab file
	exec rm K_noncoding_out.tab
	
	# Return from RESULTS to Intron_Data directory
	cd ..
	# Return from Intron_Data directory to main directory
	cd ..

	puts "\nVariables open in INTRONS18_Kchromosome : [info local]"

return
}
